<?php
/**
 * ORM-Object to handle articles from Open Content
 */

namespace EwTools\Contenttypes;

use OcAPI;
use OcArticle;
use EwTools\Handler\OpenContent;
use EwTools\Support\Collection;
use WP_Post;

/**
 * Article
 *
 * @todo    Use handler for Open Content querybuilder
 * @link    http://infomaker.se
 * @package EwTools\Contenttypes
 * @since   EwTools\Contenttypes\Article 0.1
 */
abstract class Article extends Contenttype {
    
    /**
     * Contains the type it represents in OC
     *
     * @var string
     */
    protected $contenttype = "Article";
    
    /**
     * Contains the instance of the OC article
     *
     * @var OcArticle
     */
    protected $oc_article;
    
    /**
     * Contains options for the article.
     *
     * @var array
     */
    private $options;
    
    /**
     * Contains the properties to ask for when fetching a single article
     *
     * @var array
     */
    protected $fields = [
    
    ];
    
    /**
     * EWC_Article constructor.
     *
     * @since 0.1
     *
     * @param OcArticle $oc_article
     * @param array     $options
     */
    public function __construct( \OcArticle $oc_article, $options = [] ) {
        $this->oc_article = $oc_article;
        $this->options    = $options;
    }
    
    /**
     * Function retrieve the articles permalink
     *
     * @since 0.1
     * @return mixed
     */
    protected function permalink() {
        return $this->oc_article->get_permalink();
    }
    
    /**
     * Fetch the data from the OcArticle.
     * Support multi selectors for fallback support
     *
     * @param string $selector
     *
     * @uses  OcArticle
     * @since 0.1
     * @return string
     */
    protected function getArticleField( $selector ) {
        
        $multi_selector = preg_split( '/\s*(,|\|)\s*/', $selector );
        if( ! is_array( $multi_selector ) ) {
            return $this->oc_article->get_value( $selector );
        }
        
        foreach ( $multi_selector as $i ) {
            $value = $this->oc_article->get_value( $i );
            if( ! Str::isEmpty( $value ) ) {
                return $value;
            }
        }
        
        return '';
    }
    
    /**
     * Walk through templates fields and fetch the data from the ocArticle
     *
     * @since 0.1
     * @return void
     */
    protected function setTemplateFields() {
        foreach ( $this->fields as $name => $value ) {
            $this->set( $name, $this->getArticleField( $value ) );
        }
    }
    
    /**
     * Function to fetch articles and creating a collection of the requestad article
     * TODO: not finished Use OpenContent class for fetching articles
     *
     * @param string $query
     * @param int    $limit
     *
     * @since 0.1
     * @return Collection
     */
    public static function query( $query, $limit = null ) {
        $articles      = [];
        $search_result = OpenContent::search( [
            'query'      => $query,
            'use_raw'    => true,
            'sorting'    => 'Publiceringsdag',
            'properties' => [],
            'limit'      => $limit,
            'use_cache'  => true
        ] );
        
        if( is_array( $search_result ) ) {
            unset( $search_result[ 'facet' ] );
            unset( $search_result[ 'hits' ] );
            unset( $search_result[ 'duration' ] );
            
            $articles = array_map( function ( $article ) {
                return ( new static( $article ) );
            }, $search_result );
        }
        
        return new Collection( $articles );
    }
    
    /**
     * Function fetch an article and create an instance of requested class
     *
     * @param string $uuid
     *
     * @since 0.1
     * @return null|static
     */
    public static function create( $uuid ) {
        $article = OpenContent::getArticle( $uuid );
        if( $article instanceof OcArticle ) {
            
            return new static( $article );
        }
        
        return null;
    }
    
    /**
     * Function create an article from the post type
     *
     * @param WP_Post $post
     *
     * @since    0.1
     * @return null|static
     */
    public static function createFromPost( WP_Post $post ) {
        return static::create( get_post_meta( $post->ID, 'oc_uuid', true ) );
    }
}