<?php
/**
 * Handler: to manage Open Content requests
 */
namespace EwTools\Handler;

use EwTools\Support\Str;
use \OCAPI;

/**
 * OpenContent
 *
 * @link    http://infomaker.se
 * @package EwTools\Handler
 * @since   EwTools\Handler\OpenContent 0.1
 */
class OpenContent {
    
    /**
     * The prefix for every cached search response
     *
     * @var string
     */
    protected static $cache_prefix = 'everyware_oc_search';
    
    /**
     * Default values for making text search requests against the OCAPI class
     *
     * @var array
     */
    protected static $default_search_data = [
        'query'                   => '',
        'section'                 => '',
        'sorting'                 => '',
        'start'                   => 0,
        'pubdate_start'           => '',
        'pubdate_stop'            => '',
        'limit'                   => 10,
        'include_future_articles' => false,
        'facet'                   => null,
        'facet_limit'             => null,
        'facet_mincount'          => null,
        'use_cache'               => true,
        'create_article'          => true,
        'use_raw'                 => false,
        'properties'              => null,
    ];
    
    /**
     * Default values for fetching suggestions from OCAPI class
     *
     * @var array
     */
    protected static $default_suggest_data = [
        'field'                => null,
        'incompleteWord'       => null,
        'incompleteWordInText' => null,
        'q'                    => null,
    ];
    
    /**
     * Instance of this class
     *
     * @var OpenContent
     */
    protected static $instance = null;
    
    /**
     * Instance of the OCAPI class provided by the Every plugin
     *
     * @var OCApi
     */
    private $oc_api;
    
    /**
     * Contains the query before finished
     *
     * @var string
     */
    public static $oc_query;
    
    /**
     * OpenContent constructor.
     * Uses dependency-injection
     *
     * @since 0.1
     *
     * @param OCAPI $oc_api
     */
    private function __construct( OCAPI $oc_api ) {
        $this->oc_api = $oc_api;
    }
    
    /**
     * Get instance of this class
     *
     * @since 0.1
     * @return OpenContent
     */
    public static function getInstance() {
        
        if( static::$instance === null ) {
            static::$instance = new static( new OCAPI );
        }
        
        return static::$instance;
    }
    
    /**
     * Return the requested key using the cache_prefix
     *
     * @param string $key
     *
     * @since 0.1
     * @return string
     */
    protected static function getCacheKey( $key = '' ) {
        return Str::snake( Str::prepend( static::$cache_prefix, $key, ' ' ) );
    }
    
    /**
     * Helper function for use of the OCAPI class
     *
     * @since 0.1
     * @return OCAPI
     */
    private static function ocApi() {
        return static::getInstance()->oc_api;
    }
    
    /**
     * Function to sanitize a string before being sent to OCAPI class
     *
     * @param string $value
     *
     * @since 0.1
     * @return string
     */
    private static function sanitizeQueryValue( $value = '' ) {
        return stripcslashes( trim( $value ) );
    }
    
    /**
     * Prepare OC-query by replacing some characters and restricting the search for articles
     *
     * @param string $query
     * @param bool   $raw
     *
     * @since 0.1
     * @return string
     */
    private static function prepareQuery( $query, $raw = false ) {
        
        if( ! $raw ) {
            $query = static::sanitizeQueryValue( $query );
            
            return '"' . addslashes( trim( $query ) ) . '"';
        }
        
        return $query;
    }
    
    /**
     * Parse a timestamp into the correct format fo being used in OCAPI class
     * @todo Use EwTools\Support\Date for creating OC-Date
     *
     * @param $timestamp
     *
     * @since 0.1
     * @return false|string
     */
    private static function timeToOcDate( $timestamp ) {
        return date( "Y-m-d\TH:i:s\Z", strtotime( $timestamp . ' UTC' ) );
    }
    
    /**
     * Creates a string to be added to the query for restricting the result to publish dates
     *
     * @param string $start
     * @param string $stop
     *
     * @since 0.1
     * @return string
     */
    private static function addPublishSpan( $start = '', $stop = '' ) {
        
        $date_start = '*';
        $date_stop  = 'NOW';
        
        if( $start !== '' ) {
            // What are you?
            // then convert to timestamp
            $date_start = static::timeToOcDate( $start );
        }
        
        if( $stop !== '' ) {
            // What are you?
            // then convert to timestamp
            $date_stop = static::timeToOcDate( $stop );
        }
        
        return " AND Pubdate:[" . $date_start . " TO " . $date_stop . "]";
    }
    
    /**
     * Create a query-block of all sections
     *
     * @param array $sections
     *
     * @since 0.1
     * @return string
     */
    private static function addSections( array $sections = [] ) {
        return ! empty( $sections ) ? ' AND ' . static::addPropertyBlock( $sections, 'Section' ) : '';
    }
    
    /**
     * Function for creating text-search queries against OC using OCAPI class
     *
     * @use   OCAPI
     *
     * @param array $search_data
     *
     * @since 0.1
     * @return bool
     */
    public static function search( array $search_data = [] ) {
    
        print_r( '<pre>' );
        var_dump( $search_data );
        print_r( '</pre>' );
        $search_data = array_replace( static::$default_search_data, $search_data );
        
        $query = static::prepareQuery( $search_data[ 'query' ], $search_data[ 'use_raw' ] );
        
        if( $query !== '' ) {
            
            if( ! empty( $search_data[ 'pubdate_start' ] ) || ! empty( $search_data[ 'pubdate_stop' ] ) ) {
                $query .= static::addPublishSpan( $search_data[ 'pubdate_start' ], $search_data[ 'pubdate_stop' ] );
            }
            
            $query .= static::addSections( $search_data[ 'section' ] );
            
            $sort           = $search_data[ 'sorting' ];
            $start          = $search_data[ 'start' ];
            $limit          = $search_data[ 'limit' ];
            $facet          = $search_data[ 'facet' ];
            $facet_mincount = $search_data[ 'facet_mincount' ];
            $facet_limit    = $search_data[ 'facet_limit' ];
            $create_article = $search_data[ 'create_article' ];
            $use_cache      = $search_data[ 'use_cache' ];
            $properties     = $search_data[ 'properties' ];
            
            $basic_query_criteria = $search_data[ 'include_future_articles' ] ? '' : ' AND Published:true';
            $query                = "{$basic_query_criteria} AND ({$query})";
            
            return static::ocApi()->text_search( $query, [ 'Article' ], $sort, $facet_limit, $facet_mincount, $start, $limit, $create_article, $properties, $use_cache, $facet );
        }
        
        return false;
    }
    
    /**
     * Function for fetching article suggestions
     * @todo Maybe remove?
     *
     * @uses  OCAPI
     *
     * @param array $suggest_data
     *
     * @since 0.1
     * @return array
     */
    public static function getSuggestions( array $suggest_data = [] ) {
        
        $suggest_data = array_replace( static::$default_suggest_data, $suggest_data );
        
        $field                = $suggest_data[ 'field' ];
        $q                    = $suggest_data[ 'q' ];
        $incompleteWord       = $suggest_data[ 'incompleteWord' ];
        $incompleteWordInText = $suggest_data[ 'incompleteWordInText' ];
        
        header( 'Content-type: application/json' );
        
        return static::ocApi()->ajax_get_suggest( $field, $q, $incompleteWord, $incompleteWordInText );
    }
    
    /**
     * Fetch single article from id
     * @todo do we need to cache this?
     *
     * @uses  OCAPI
     *
     * @param $uuid
     *
     * @since 0.1
     * @return bool
     */
    public static function getArticle( $uuid ) {
        
        $options_key = static::getCacheKey( $uuid );
        
        if( false === ( $article = get_transient( $options_key ) ) ) {
            
            $article = static::ocApi()->get_single_article( $uuid );
            $article = isset( $article[ 'article' ] ) ? $article[ 'article' ] : false;
            
            if( $article ) {
                set_transient( $options_key, $article, 60 * MINUTE_IN_SECONDS );
            }
        }
        
        return $article;
    }
    
    /**
     * Creates a Solr-block of properties with the option to change the seperatoer
     *
     * @param array  $properties
     * @param string $property_name
     * @param string $glue
     *
     * @since 0.1
     * @return string
     */
    public static function addPropertyBlock( array $properties, $property_name = '', $glue = 'OR' ) {
        $properties = array_unique( array_filter( $properties ) );
        if( empty( $properties ) ) {
            return '';
        }
        $glue = trim( $glue ); // Make sure that there is no space around the glue
        
        $block = implode( " {$glue} ", array_map( function ( $property ) use ( $property_name ) {
            return OpenContent::createProperty( $property, $property_name );
        }, $properties ) );
        
        return count( $properties ) > 1 ? '(' . $block . ')' : $block;
    }
    
    /**
     * Create a Solr-property string
     *
     * @uses  Str::contains()
     *
     * @param $property
     * @param $property_name
     *
     * @since 0.1
     * @return string
     */
    public static function createProperty( $property, $property_name ) {
        $property = Str::contains( $property, " " ) ? '"' . $property . '"' : $property;
        
        return "{$property_name}:{$property}";
    }
}