<?php
/**
 * This class uses Carbon for advanced date handling.
 * (c) Brian Nesbitt <brian@nesbot.com>
 *
 * @link http://carbon.nesbot.com/
 *       For the full copyright and license information, please view the LICENSE
 *       file that was distributed with this source code.
 */
namespace EwTools\Support;

use Carbon\Carbon;
use DateTimeZone;
use IntlDateFormatter;

/**
 * Date
 *
 * @link    http://infomaker.se
 * @package EwTools\Support
 * @since   EwTools\Support\Date 0.1
 */
class Date extends Carbon {
    
    /**
     * The format of a date in Open Content
     */
    const OC = 'Y-m-d\TH:i:s\Z';
    
    /**
     * Contains an instance of the formatter used to translate dates into text
     *
     * @var IntlDateFormatter|null
     */
    protected static $formatter = null;
    
    /**
     * Contains the timezone set as an environment-variable on the server
     *
     * @var string
     */
    protected static $default_timezone = null;
    
    /**
     * Contains the local/language set as an environment-variable on the server
     *
     * @var string
     */
    protected static $default_locale = null;
    
    /**
     * Date constructor.
     *
     * @since 0.1
     *
     * @param string|null              $time
     * @param DateTimeZone|string|null $tz
     */
    public function __construct( $time = null, $tz = null ) {
        static::$default_timezone = getenv( 'PHP_TIMEZONE' ) ?: 'Europe/Stockholm';
        static::$default_locale   = getenv( 'PHP_DEFAULT_LOCALE' ) ?: 'sv-SE';
        
        static::setLocale( explode( '_', Str::snake( static::$default_locale ) )[ 0 ] );
        date_default_timezone_set( static::$default_timezone );
        
        parent::__construct( $time, $tz );
    }
    
    /**
     * Convert date into a format readable to Open Content
     *
     * @since 0.1
     * @return string
     */
    public function toOcString() {
        $this->setTimezone( 'UTC' ); // Reset Timezone
        return $this->format( static::OC );
    }
    
    /**
     * Use the custom formatter to format and translate a date into the requested format
     *
     * @param $format
     *
     * @uses  IntlDateFormatter::formatObject()
     * @since 0.1
     * @return string
     */
    public function toText( $format ) {
        return static::formatter()->formatObject( $this, $format );
    }
    
    /**
     * Create a date using the format from Open Content
     *
     * @param string $date
     *
     * @since 0.1
     * @return static
     */
    public static function createFromOcString( $date ) {
        return static::createFromTimestamp( strtotime( $date ) );
    }
    
    /**
     * Function for retrieving the formatter used to format dates into human readable string
     *
     * @since 0.1
     * @return IntlDateFormatter|null
     */
    public static function formatter() {
        if( static::$formatter === null ) {
            static::setFormatter(static::$default_locale);
        }
        
        return static::$formatter;
    }
    
    /**
     * Function for retrieving the formatter used to format dates into human readable string
     *
     * @param string $local
     *
     * @since 0.1
     * @return IntlDateFormatter|null
     */
    public static function setFormatter($local='') {
        static::$formatter = new IntlDateFormatter ( $local, // string locale
            IntlDateFormatter::FULL, // int date type
            IntlDateFormatter::FULL // int time type
        );
    }
}